/*BEGIN COPYRIGHT */
//*************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999, 2001
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//************************************************************************/
/*END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */
//*************************************************************************
//
//  This application demonstrates the new logon methods of
//  MQSeries Workflow using credentials and the Server Authentication Exit.
//
//*************************************************************************

package com.ibm.workflow.java.exit.sample;

// import necessary definitions of the MQWF API
import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ServicePackage.*;
import com.ibm.workflow.api.ProcessInstancePackage.*;

// import necessary java packages
import java.io.*;
import java.beans.PropertyVetoException;
import java.security.cert.*;

public class AuthenticationClient
{
  // This class is used to demonstrate the different possibilities for user logons to MQWF
  // Especially the new authetication exit of Release V330 is tested via calls to the logon3
  // or logon4 methods of the ExecutionServer class.
  // The used examples for username/password pairs are similar to those those used by the C Example
  // for the Authentication exit.

  public static void main(String[] args)
  {
    String certificateFileName = "";
    String certificateType = "";
    ExecutionService service = null;

    if (args.length != 0)
    {
      // if parameters are specified the first is the filename
      // of the file containing the certficate
      certificateFileName = args[0];
      if (args.length >= 2)
      {
        // if a second parameter is specified it describes the type of the certificate.
        // Attention: The sample implementation of the server authentication exit can only handle
        // X509 certificates or objects of the Credentials class!
        // If you want to use other certificate types, you have to implement your own exit!
        certificateType = args[1];
      }
      else
      {
        certificateType = "X509";
      }
    }
    else
    {
      // use the default file provided with the sample
      // which contains an X509 certificate
      certificateFileName = "AdminCert.cert";
      certificateType= "X509";
    }
    // In order to do a logon, a connection to the MQSeries Workflow Admin Server must be established.
    // First get an Agent object to set the locator policy: here Loc_Locator is used because it uses
    // an instance of an embedded agent deamon, which does not require special configuration
    try
    {
      Agent myAgent = new Agent();
      myAgent.setLocator(Agent.LOC_LOCATOR);
      myAgent.setName("MQWFAGENT");
      // By using empty strings for Systemgroup and System in the following call, the service returned
      // is the default service of the configuration.
      service = myAgent.locate("","");
      if (service == null)
      {
        System.out.println("Service is null! Cannot proceed.");
        System.exit(1);
      }
    }
    catch (FmcException fmcx)
    {
      System.out.println("FMC: Error getting execution service object");
      fmcx.printStackTrace();
      System.exit(1);
    }
    catch (PropertyVetoException pvx)
    {
      System.out.println("PropertyVetoException occured setting locator policy!");
      pvx.printStackTrace();
      System.exit(1);
    }

    // After the service object is available, the logons can proceed
    // First test an log on using a X509 Certificate and logon3()
    try
    {
      System.out.println();
      System.out.println("Using a certificate for user 'Administrator' who is authorized");
      System.out.println("The sample implementation will map the certificate to Workflow user 'ADMIN'");
      System.out.println();
      System.out.println("-------------------------------------------------------------------------------");
      System.out.println("Using Certificate stored in: " + certificateFileName);
      System.out.println(" Certificate Type: " + certificateType);
      System.out.println();
      FileInputStream fi = new FileInputStream(certificateFileName);
      CertificateFactory cf = CertificateFactory.getInstance(certificateType);
      X509Certificate c = (X509Certificate) cf.generateCertificate(fi);
      System.out.println("Read in the following certificate:");
      System.out.println("\tCertificate for: " + c.getSubjectDN());
      System.out.println("\tCertificate issued by: " + c.getIssuerDN());
      System.out.println("\tThe certificate is valid from " +
                         c.getNotBefore() + " to " + c.getNotAfter());
      System.out.println("\tCertificate SN# " + c.getSerialNumber());
      System.out.println("\tGenerated with " + c.getSigAlgName());
      byte[] test = c.getEncoded();
      service.logon3(test);
      // If you like to test the more complex logon4 method make the previous line a comment and
      // uncomment the following line!
      //service.logon4(test, SessionMode.from_int(SessionMode._DEFAULT), AbsenceIndicator.from_int(AbsenceIndicator._NOT_SET), "");
      if (service.isLoggedOn())
      {
        System.out.println("Logon successful!");
      }
      else
      {
        System.out.println("Logon not successful!");
      }
    }
    catch (FmcException fmcex)
    {
      System.out.println("FMCException occurred while trying to log on!");
    }
    catch (Throwable e)
    {
      System.out.println("Exception occurred during logon process!");
      e.printStackTrace();
    }
    try
    {
      if (service.isLoggedOn())
      {
        // If the first logon was successful, another logon results in an AlreadyLoggedOn-Error
        // therefor the user must log off first!
        service.logoff();
      }
    }
    catch (FmcException fmcx)
    {
      System.out.println("Could not log off successfully!");
    }
    System.out.println();
    System.out.println("Now trying user defined credential");
    System.out.println("-------------------------------------------------------------------------------");
    System.out.println("The User ID stored in the user defined Credentials class is not authorized!");
    System.out.println("Therefore the logon attempt will be denied. Check the sample code for details");
    System.out.println("-------------------------------------------------------------------------------");
    System.out.println();
    // Now demonstrate the use of a user defined byte array in the logon3() method. The sample Credentials class stores a user ID, that is not
    // authenticated, therefore the second logon attempt fails.
    Credentials myCred = new Credentials();
    try
    {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      // use the ByteOutputStream to serialize the credential
      ObjectOutputStream serialize = new ObjectOutputStream(out);
      serialize.writeObject(myCred);
      serialize.flush();
      serialize.close();
      out.close();
      // now logon to MQWF with the written credential
      service.logon3(out.toByteArray());
      // If you like to test the more complex logon4 method make the previous line a comment and
      // uncomment the following line!
      //service.logon4(out.toByteArray(), SessionMode.from_int(SessionMode._DEFAULT), AbsenceIndicator.from_int(AbsenceIndicator._NOT_SET), "");
      if (service.isLoggedOn())
      {
        System.out.println("Logon successful!");
      }
      else
      {
        System.out.println("Logon not successful!");
      }
    }
    catch (IOException iox)
    {
      System.out.println("Error writing object to ByteArrayOutputStream!");
    }
    catch (FmcException fmcx)
    {
      System.out.println("Error logging on to MQWF");
      fmcx.printStackTrace();
      }
    try
    {
      if (service.isLoggedOn())
      {
        // If the logon was successful, another logon results in an AlreadyLoggedOn-Error
        // therefor the user must log off first!
        service.logoff();
      }
    }
    catch (FmcException fmcx)
    {
      System.out.println("Could not log off successfully!");
    }

  }

}
